﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.LiveFX.Client;

namespace LiveFrameworkExtensions
{
    public static class DataEntryExtensions
    {
        public static bool IsRootEntry(this DataEntry entry)
        {
            if (String.IsNullOrEmpty(entry.Resource.ParentId))
                return true;

            return entry.Resource.ParentId.Equals(MeshConstants.RootDataEntryId);
        }

        public static DataFeed GetDataFeed(this DataEntry entry)
        {
            return (from feed in
                        (from meshobj in entry.LiveOperatingEnvironment.Mesh.MeshObjects.Entries
                         select meshobj.DataFeeds.Entries).Aggregate((fl1, fl2) => fl1.Union(fl2))
                    where feed.DataEntries.Entries.Contains(entry)
                    select feed).Single();
        }

        public static ICollection<DataEntry> GetChildEntries(this DataEntry entry)
        {
            return (from child in entry.GetDataFeed().DataEntries.Entries
                    where child.Resource.ParentId == entry.Resource.Id
                    select child).ToList();
        }

        public static ICollection<DataEntry> GetFileChildEntries(this DataEntry entry)
        {
            return (from child in entry.GetChildEntries()
                    where child.Resource.Type == DataEntryTypes.File
                    select child).ToList();
        }

        public static ICollection<DataEntry> GetFolderChildEntries(this DataEntry entry)
        {
            return (from child in entry.GetChildEntries()
                    where child.Resource.Type == DataEntryTypes.Folder
                    select child).ToList();
        }
    }
}
