﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.LiveFX.Client;
using System.Collections.ObjectModel;

namespace LiveFrameworkExtensions
{
    public static class DataFeedExtensions
    {
        public static ICollection<DataEntry> GetRootDataEntries(this DataFeed feed)
        {
            return (from entry in feed.DataEntries.Entries
                    where entry.IsLoaded && entry.IsRootEntry()
                    select entry).ToList();
        }

        public static MeshObject GetMeshObject(this DataFeed feed)
        {
            return (from meshobject in feed.LiveOperatingEnvironment.Mesh.MeshObjects.Entries
                    where meshobject.DataFeeds.Entries.Contains(feed)
                    select meshobject).Single();
        }

        public static ICollection<DataEntry> GetFileDataEntries(this DataFeed feed)
        {
            return feed.GetFileDataEntries(false);
        }

        public static ICollection<DataEntry> GetFolderDataEntries(this DataFeed feed)
        {
            return feed.GetFolderDataEntries(false);
        }

        public static ICollection<DataEntry> GetFileDataEntries(this DataFeed feed, bool onlyRootEntries)
        {
            var entries = onlyRootEntries ? feed.GetRootDataEntries() : feed.DataEntries.Entries;
            return (from entry in entries
                    where entry.Resource.Type == DataEntryTypes.File
                    select entry).ToList();
        }

        public static ICollection<DataEntry> GetFolderDataEntries(this DataFeed feed, bool onlyRootEntries)
        {
            var entries = onlyRootEntries ? feed.GetRootDataEntries() : feed.DataEntries.Entries;
            return (from entry in entries
                    where entry.Resource.Type == DataEntryTypes.Folder
                    select entry).ToList();
        }
    }
}
