﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

using System.Net;
using Microsoft.LiveFX.Client;
using Microsoft.LiveFX.Client.Notification;
using Microsoft.LiveFX.ResourceModel;
using Microsoft.LiveFX.ResourceModel.Description;
using Microsoft.LiveFX.ResourceModel.Query;
using Microsoft.LiveFX.ResourceModel.Scripting;
using Microsoft.LiveFX.ResourceModel.Scripting.Runtime;

using LiveFrameworkExtensions;


namespace WinFormsApp
{
    public partial class MeshMainForm : Form
    {
        LiveOperatingEnvironment _loe;

        public MeshMainForm()
        {
            InitializeComponent();

            bool connected = false;
            bool close = false;
            bool showForm = MyCreds.IsNotComplete;
            while(!connected && !close)
            {
                if(showForm)
                {
                    var credForm = new CredentialsForm(MyCreds.Username, MyCreds.Password);
                    if (credForm.ShowDialog() != DialogResult.OK)
                    {
                        close = true;
                    }
                    else
                    {
                        MyCreds.Username = credForm.Username;
                        MyCreds.Password = credForm.Password;
                    }
                }

                if (!close)
                {
                    var thread = new Thread(new ThreadStart(() => (new ConnectingForm()).ShowDialog()));
                    thread.Start();
                    try
                    {
                        var creds = new NetworkCredential(MyCreds.Username, MyCreds.Password, "https://user-ctp.windows.net");
                        _loe = new LiveOperatingEnvironment();

                        //loe.ConnectLocal();
                        _loe.Connect(creds);

                        connected = true;
                    }
                    catch
                    {
                        thread.Abort();
                        MessageBox.Show("Could not connect to the cloud LOE...", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        
                        if(!showForm)
                            close = true;
                    }
                    thread.Abort();
                }
            }

            if (close)
            {
                this.Close();
                return;
            }
            else
            {
                treeViewDevices.Nodes.Add(new TreeNode("Devices"));
                treeViewNews.Nodes.Add(new TreeNode("News"));
                treeViewMeshObjects.Nodes.Add(new TreeNode("MeshObjects"));
                treeViewContacts.Nodes.Add(new TreeNode("Contacts"));

                if (_loe.IsRunning())
                {
                    foreach (var entry in _loe.Mesh.Devices.Entries)
                        treeViewDevices.Nodes[0].Nodes.Add(new TreeNode(entry.Resource.Title));
                    foreach (var entry in _loe.Mesh.News.Entries)
                    {
                        treeViewNews.Nodes[0].Nodes.Add(new TreeNode(entry.Resource.Title));
                    }

                    foreach (var meshobj in _loe.Mesh.MeshObjects.Entries)
                    {
                        var meshobjNode = new TreeNode("MeshObject: " + meshobj.Resource.Title + " (" + meshobj.Resource.Type + ")");
                        treeViewMeshObjects.Nodes[0].Nodes.Add(meshobjNode);
                        foreach (var feed in meshobj.DataFeeds.Entries)
                        {
                            var feedNode = new TreeNode("DataFeed: " + feed.Resource.Title);
                            meshobjNode.Nodes.Add(feedNode);
                            foreach (var rootEntry in feed.GetRootDataEntries())
                            {
                                var rootNode = new TreeNode(rootEntry.Resource.Title + " (" + rootEntry.Resource.Type + ")");
                                feedNode.Nodes.Add(rootNode);
                                addTreeNodesForChildDataEntries(rootEntry, rootNode);
                            }
                        }
                    }
                    if (!_loe.IsLocalConnection)
                    {
                        foreach (var entry in _loe.Contacts.Entries)
                            treeViewContacts.Nodes[0].Nodes.Add(new TreeNode(entry.Resource.Title));
                    }
                }

                treeViewDevices.ExpandAll();
                treeViewNews.ExpandAll();
                treeViewMeshObjects.ExpandAll();
                treeViewContacts.ExpandAll();

                loadMeshObjectsListbox();
            }
        }

        private void addTreeNodesForChildDataEntries(DataEntry parentEntry, TreeNode parentNode)
        {
            foreach (var childEntry in parentEntry.GetChildEntries())
            {
                var childNode = new TreeNode(childEntry.Resource.Title + ": " + childEntry.Resource.Type);
                parentNode.Nodes.Add(childNode);
                addTreeNodesForChildDataEntries(childEntry, childNode);
            }
        }

        private void loadMeshObjectsListbox()
        {
            listBoxMeshObjects.Items.Clear();
            if (_loe.IsRunning())
            {
                foreach (var entry in _loe.Mesh.MeshObjects.Entries)
                {
                    listBoxMeshObjects.Items.Add(entry);
                }
            }
        }

        private void loadDataFeedsListbox()
        {
            listBoxDataFeeds.Items.Clear();
            if (_loe.IsRunning() && listBoxMeshObjects.SelectedItem != null)
            {
                MeshObject obj = listBoxMeshObjects.SelectedItem as MeshObject;
                foreach (var feed in obj.DataFeeds.Entries)
                {
                    listBoxDataFeeds.Items.Add(feed);
                }
            }
        }

        private void loadDataEntriesListbox()
        {
            listBoxDataEntries.Items.Clear();
            if (_loe.IsRunning() && listBoxDataFeeds.SelectedItem != null)
            {
                DataFeed feed = listBoxDataFeeds.SelectedItem as DataFeed;
                foreach (var entry in feed.DataEntries.Entries)
                {
                    listBoxDataEntries.Items.Add(entry);
                }
            }
        }

        private void listBoxMeshObjects_SelectedIndexChanged(object sender, EventArgs e)
        {
            loadDataFeedsListbox();
            listBoxDataEntries.Items.Clear();
        }

        private void listBoxDataFeeds_SelectedIndexChanged(object sender, EventArgs e)
        {
            loadDataEntriesListbox();
        }

        private void listBoxDataEntries_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && listBoxDataEntries.SelectedItem != null)
            {
                var entry = listBoxDataEntries.SelectedItem as DataEntry;
                try
                {
                    pictureBox1.Image = new Bitmap(entry.ReadMediaResource());
                    pictureBox1.SizeMode = PictureBoxSizeMode.Zoom;
                }
                catch { }
            }
        }

        private void buttonNewMeshObject_Click(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && textBoxMeshObject.Text.Trim() != String.Empty)
            {
                var obj = new MeshObject(textBoxMeshObject.Text);
                _loe.Mesh.MeshObjects.Add(ref obj);
                
                loadMeshObjectsListbox();
                textBoxMeshObject.Text = String.Empty;
            }
        }

        private void buttonUpdateMeshObject_Click(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && listBoxMeshObjects.SelectedItem != null && textBoxMeshObject.Text.Trim() != String.Empty)
            {
                var obj = listBoxMeshObjects.SelectedItem as MeshObject;
                obj.Resource.Title = textBoxMeshObject.Text;
                obj.Update();
                
                loadMeshObjectsListbox();
                textBoxMeshObject.Text = String.Empty;
            }
        }

        private void buttonDeleteMeshObject_Click(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && listBoxMeshObjects.SelectedItem != null)
            {
                var obj = listBoxMeshObjects.SelectedItem as MeshObject;
                _loe.Mesh.MeshObjects.Remove(obj);
                
                listBoxDataFeeds.Items.Clear();
                listBoxDataEntries.Items.Clear();
                loadMeshObjectsListbox();
            }
        }

        private void buttonNewDataFeed_Click(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && listBoxMeshObjects.SelectedItem != null && textBoxDataFeed.Text.Trim() != String.Empty)
            {
                var obj = listBoxMeshObjects.SelectedItem as MeshObject;
                var feed = new DataFeed(textBoxDataFeed.Text);
                obj.DataFeeds.Add(ref feed);
                
                loadDataFeedsListbox();
                textBoxDataFeed.Text = String.Empty;
            }
        }

        private void buttonUpdateDataFeed_Click(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && listBoxDataFeeds.SelectedItem != null && textBoxDataFeed.Text.Trim() != String.Empty)
            {
                var feed = listBoxDataFeeds.SelectedItem as DataFeed;
                feed.Resource.Title = textBoxDataFeed.Text;
                feed.Update();
                
                loadDataFeedsListbox();
                textBoxDataFeed.Text = String.Empty;
            }
        }

        private void buttonDeleteDataFeed_Click(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && listBoxMeshObjects.SelectedItem != null && listBoxDataFeeds.SelectedItem != null)
            {
                var obj = listBoxMeshObjects.SelectedItem as MeshObject;
                var feed = listBoxDataFeeds.SelectedItem as DataFeed;
                obj.DataFeeds.Remove(feed);

                listBoxDataEntries.Items.Clear();
                loadDataFeedsListbox();
            }
        }

        private void buttonNewDataEntry_Click(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && listBoxDataFeeds.SelectedItem != null && textBoxDataEntry.Text.Trim() != String.Empty)
            {
                var feed = listBoxDataFeeds.SelectedItem as DataFeed;
                var entry = new DataEntry(textBoxDataEntry.Text);
                feed.DataEntries.Add(ref entry);
                
                loadDataEntriesListbox();
                textBoxDataEntry.Text = String.Empty;
            }
        }

        private void buttonUpdateDataEntry_Click(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && listBoxDataFeeds.SelectedItem != null && textBoxDataEntry.Text.Trim() != String.Empty)
            {
                var entry = listBoxDataEntries.SelectedItem as DataEntry;
                entry.Resource.Title = textBoxDataEntry.Text;
                entry.Update();
                
                loadDataEntriesListbox();
                textBoxDataEntry.Text = String.Empty;
            }
        }

        private void buttonDeleteDataEntry_Click(object sender, EventArgs e)
        {
            if (_loe.IsRunning() && listBoxDataFeeds.SelectedItem != null && listBoxDataEntries.SelectedItem != null)
            {
                var feed = listBoxDataFeeds.SelectedItem as DataFeed;
                var entry = listBoxDataEntries.SelectedItem as DataEntry;
                feed.DataEntries.Remove(entry);
                
                loadDataEntriesListbox();
            }
        }
    }
}
